@@ -128,7 +128,7 @@ class StubsGenerator(object):
 
     @staticmethod
     def fully_qualified_name(klass):
-        module_name = klass.__module__ if hasattr(klass,'__module__') else None
+        module_name = klass.__module__ if hasattr(klass, '__module__') else None
         class_name = klass.__qualname__
 
         if module_name == "builtins":
@@ -289,11 +289,18 @@ class AttributeStubsGenerator(StubsGenerator):
         value_lines = repr(self.attr).split("\n")
         if len(value_lines) == 1:
             value = value_lines[0]
+            # remove random address from <foo.Foo object at 0x1234>
+            value = re.sub(r" at 0x[0-9a-fA-F]+>", ">", value)
+            typename = self.fully_qualified_name(type(self.attr))
+            if value == "<{typename} object>".format(typename=typename):
+                value_comment = ""
+            else:
+                value_comment = " # value = {value}".format(value=value)
             return [
-                "{name}: {typename} # value = {value}".format(
+                "{name}: {typename}{value_comment}".format(
                     name=self.name,
-                    typename=self.fully_qualified_name(type(self.attr)),
-                    value=value)
+                    typename=typename,
+                    value_comment=value_comment)
             ]
         else:
             return [
@@ -415,7 +422,7 @@ class PropertyStubsGenerator(StubsGenerator):
 
 
 class ClassStubsGenerator(StubsGenerator):
-    ATTRIBUTES_BLACKLIST = ("__class__", "__module__", "__qualname__")
+    ATTRIBUTES_BLACKLIST = ("__class__", "__module__", "__qualname__", "__dict__", "__weakref__", "__annotations__")
     METHODS_BLACKLIST = ("__dir__", "__sizeof__")
     BASE_CLASS_BLACKLIST = ("pybind11_object", "object")
     CLASS_NAME_BLACKLIST = ("pybind11_type",)
@@ -453,7 +460,18 @@ class ClassStubsGenerator(StubsGenerator):
             return
         _visited_objects.append(self.klass)
 
+        bases = inspect.getmro(self.klass)[1:]
+
+        def is_base_member(name, member):
+            for base in bases:
+                if hasattr(base, name) and getattr(base, name) is member:
+                    return True
+            return False
+
         for name, member in inspect.getmembers(self.klass):
+            # check if attribute is in __dict__ (fast path) before slower search in base classes
+            if name not in self.klass.__dict__ and is_base_member(name, member):
+                continue
             if inspect.isroutine(member):
                 self.methods.append(ClassMemberStubsGenerator(name, member, self.klass.__module__))
             elif name != '__class__' and inspect.isclass(member):
@@ -473,8 +491,6 @@ class ClassStubsGenerator(StubsGenerator):
                                  self.fields):
             x.parse()
 
-        bases = inspect.getmro(self.klass)[1:]
-
         for B in bases:
             if B.__name__ != self.klass.__name__ and B.__name__ not in self.base_class_blacklist:
                 self.base_classes.append(B)
